#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <glsc.h>

#define CAR_ARRAY 1001
#define NCARS 50
#define LENGTH 100.0

#define A     1.0
#define ova 2.0
#define ovb 4.0
#define ovc 2.0

#define DEL 0.1

double ov(double dx){
  return ova*(1/(1+exp(-ovb*(dx-ovc)))-1/(1+exp(ovb*ovc)));
}

double d_ov(double dx){
  return log(1.0+DEL*DEL*ov(dx))/DEL/DEL;
}

double d_v(double v){
  return log(1+DEL*(exp(v)-1))/DEL/DEL;
}

#define WINDOW_WIDTH_MM  250.0
#define WINDOW_HEIGHT_MM 120.0

#define COORD_LEFT_MM1   20.0
#define COORD_TOP_MM1    30.0
#define COORD_WIDTH_MM1  80.0
#define COORD_HEIGHT_MM1 80.0

#define COORD_LEFT_MM2   130.0
#define COORD_TOP_MM2    45.0
#define COORD_WIDTH_MM2  100.0
#define COORD_HEIGHT_MM2 60.0

#define NGRAPH 100
#define RAND_AMP 0.01
#define BIG_INT  1000000
#define INIT_WAIT_SEC 0.01
#define DRAW_INTERVAL 1
#define RECORD_POINTS 100
#define RECORD_POINTS_INTERVAL 5

main(){
  int k, ncars, n;
  double length;
  double x[CAR_ARRAY][3];
  double x_min_snap, x_max_snap, y_min_snap, y_max_snap;
  double x_min_phase, x_max_phase, y_min_phase, y_max_phase;
  double phase_record[RECORD_POINTS][2];
  int i;
  double theta;
  char str[256];
  double del_x_graph;
  double wait_sec=INIT_WAIT_SEC;
  g_mouse_info m_info;
  int draw_interval = DRAW_INTERVAL;

  ncars=NCARS;
  length=LENGTH;

  x_min_snap  = -1.0;
  x_max_snap  =  1.0;
  y_min_snap  = -1.0;
  y_max_snap  =  1.0;
  x_min_phase =  0.0;
  x_max_phase =  4.0;
  y_min_phase =  0.0;
  y_max_phase =  2.2;
  del_x_graph = x_max_phase/NGRAPH;

  g_init("meta", WINDOW_WIDTH_MM, WINDOW_HEIGHT_MM);
  g_device(G_DISP);
  g_def_scale(0, x_min_snap, x_max_snap, y_min_snap, y_max_snap,
    COORD_LEFT_MM1, COORD_TOP_MM1, COORD_WIDTH_MM1, COORD_HEIGHT_MM1);
  g_def_scale(1, x_min_phase, x_max_phase, y_min_phase, y_max_phase,
    COORD_LEFT_MM2, COORD_TOP_MM2, COORD_WIDTH_MM2, COORD_HEIGHT_MM2);

  g_line_width(0);
  g_cls();
  g_set_font(G_FONT_GOTHIC, 14.0, G_FONT_PLAIN);

  srand((unsigned)time(NULL));

  for(k=0; k<ncars; k++){
    x[k][0] = length/ncars*(k + (rand()%BIG_INT)/(double)BIG_INT*RAND_AMP);
    x[k][1] = x[k][0];
  }
  x[ncars][0] = x[0][0] + length;
  x[ncars][1] = x[0][1] + length;

  for(i=0; i<RECORD_POINTS; i++){
    phase_record[i][0] = x[1][0]-x[0][0];
    phase_record[i][1] = d_v(x[0][1]-x[0][0]);
  }

  for(n=1; ; n++){
    for(k=0; k<ncars; k++){
      x[k][2] = 2*x[k][1]-x[k][0]
      + DEL*DEL*A*(d_ov(x[k+1][1]-x[k][1]) - d_v(x[k][1]-x[k][0]));
    }

    x[ncars][2] = x[0][2] + length;

    if(n%RECORD_POINTS_INTERVAL == 0){
      for(i=RECORD_POINTS-1; i>0; i--){
        phase_record[i][0] = phase_record[i-1][0];
        phase_record[i][1] = phase_record[i-1][1];
      }
      phase_record[0][0] = x[1][0]-x[0][0];
      phase_record[0][1] = d_v(x[0][1]-x[0][0]);
    }

    if(n%draw_interval == 0){
      g_sleep(wait_sec);
      g_new_frame();
      g_sel_scale(0);

      m_info = g_mouse();
      if(m_info.lbutton == G_YES){
        if(m_info.y > 0.0){
          draw_interval++;
//          wait_sec = wait_sec/2;
        }
        else{
          if(draw_interval>1){
            draw_interval--;
          }
//          wait_sec = 2*wait_sec;
        }
      }

      g_line_color(G_BLACK);
      g_circle(0.0, 0.0, 1.0, G_YES, G_NO);
    
      g_line_color(G_RED);
      g_area_color(G_RED);
      for(k=0; k<ncars; k++){
        theta =(x[k][0]/length-(int)(x[k][0]/length))*2*M_PI;
        g_circle(cos(theta), sin(theta), 0.03, G_YES, G_YES);
      }

      sprintf(str, "discrete OV model  cars=%d, average distance=%.1f", ncars, length/ncars);
      g_text(COORD_LEFT_MM1, COORD_TOP_MM1-20.0, str);
      sprintf(str, "A=%.2f, a=%.2f, b=%.2f, c=%.2f, =%.2f", A, ova, ovb, ovc, DEL);
      g_text(COORD_LEFT_MM1, COORD_TOP_MM1-10.0, str);

      g_sel_scale(1);
      g_line_color(G_BLACK);
      g_move(x_min_phase, y_max_phase);
      g_plot(x_min_phase, y_min_phase);
      g_plot(x_max_phase, y_min_phase);
      g_text_s(x_max_phase, 0.0, "x");
      g_text_s(0.0, y_max_phase, "v");


      g_move(0.0, ov(0.0)/DEL/DEL);
      for(i=1; i<=NGRAPH; i++){
        g_plot(i*del_x_graph, d_ov(i*del_x_graph));
      }

      g_line_color(G_GREEN);
      g_move(phase_record[0][0], phase_record[0][1]);

      for(i=1; i<RECORD_POINTS; i++){
        g_plot(phase_record[i][0], phase_record[i][1]);
      }

      g_line_color(G_BLUE);
      g_area_color(G_BLUE);
      for(k=0; k<ncars; k++){
        g_circle(x[k+1][0]-x[k][0], d_v(x[k][1]-x[k][0]), 0.03, G_YES, G_YES);
      }
      g_show_frame();
    }

    for(k=0; k<=ncars; k++){
      x[k][0] = x[k][1];
      x[k][1] = x[k][2];
    }
  }

  g_sleep(-1.0);
  g_term();
}
