#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <glsc.h>

#define WINDOW_WIDTH_MM  230.0
#define WINDOW_HEIGHT_MM 120.0

#define COORD_LEFT_MM1   10.0
#define COORD_TOP_MM1    30.0
#define COORD_WIDTH_MM1  80.0
#define COORD_HEIGHT_MM1 80.0

#define COORD_LEFT_MM2   120.0
#define COORD_TOP_MM2    45.0
#define COORD_WIDTH_MM2  100.0
#define COORD_HEIGHT_MM2 60.0

#define DEL   0.1
#define DT    0.01
#define A     1.0
#define ova 2.0
#define ovb 4.0
#define ovc 2.0

#define CAR_ARRAY 1001
#define NCARS 50
#define LENGTH 100.0

#define NGRAPH 100
#define RAND_AMP 0.01
#define BIG_INT  1000000
#define INIT_WAIT_SEC 0.0
#define DRAW_INTERVAL 1
#define RECORD_POINTS 1000
#define RECORD_POINTS_INTERVAL 5

extern double ov(), rk_x(), rk_v();

//int koma=0;

main(){
  int n, ncars, t;
  double b, length;
  double x[CAR_ARRAY], v[CAR_ARRAY], kx[4][CAR_ARRAY], kv[4][CAR_ARRAY];
  double x_min_snap, x_max_snap, y_min_snap, y_max_snap;
  double x_min_phase, x_max_phase, y_min_phase, y_max_phase;
  double phase_record[RECORD_POINTS][2];
  int i;
  double theta;
  char str[256];
  double del_x_graph;
  double wait_sec=INIT_WAIT_SEC;
  g_mouse_info m_info;
  int draw_interval = DRAW_INTERVAL;

  ncars=NCARS;
  length=LENGTH;

  b = length/ncars;

  x_min_snap  = -1.0;
  x_max_snap  =  1.0;
  y_min_snap  = -1.0;
  y_max_snap  =  1.0;
  x_min_phase =  0.0;
  x_max_phase =  4.5;
  y_min_phase =  0.0;
  y_max_phase =  2.5;
  del_x_graph = x_max_phase/NGRAPH;

  g_init("meta", WINDOW_WIDTH_MM, WINDOW_HEIGHT_MM);
  g_device(G_DISP);
  g_def_scale(0, x_min_snap, x_max_snap, y_min_snap, y_max_snap,
    COORD_LEFT_MM1, COORD_TOP_MM1, COORD_WIDTH_MM1, COORD_HEIGHT_MM1);
  g_def_scale(1, x_min_phase, x_max_phase, y_min_phase, y_max_phase,
    COORD_LEFT_MM2, COORD_TOP_MM2, COORD_WIDTH_MM2, COORD_HEIGHT_MM2);

  g_line_width(0);
  g_cls();
  g_set_font(G_FONT_GOTHIC, 14.0, G_FONT_PLAIN);

  srand((unsigned)time(NULL));

  for(n=0; n<ncars; n++){
    x[n] = b*(n + (rand()%BIG_INT)/(double)BIG_INT*RAND_AMP);
    v[n] = 0.0;
  }
  x[ncars] = x[0] + length;
  v[ncars] = v[0];

  for(i=0; i<RECORD_POINTS; i++){
    phase_record[i][0] = x[1]-x[0];
    phase_record[i][1] = v[0];
  }

  for(t=1; ; t++){
    for(n=0; n<ncars; n++){
      kx[0][n] = rk_x(x[n+1], x[n], v[n]);
      kv[0][n] = rk_v(x[n+1], x[n], v[n]);
    }
    kx[0][ncars] = kx[0][0];

    for(n=0; n<ncars; n++){
      kx[1][n] = rk_x(x[n+1]+DT/2*kx[0][n+1], x[n]+DT/2*kx[0][n],
                      v[n]+DT/2*kv[0][n]);
      kv[1][n] = rk_v(x[n+1]+DT/2*kx[0][n+1], x[n]+DT/2*kx[0][n],
                      v[n]+DT/2*kv[0][n]);
    }
    kx[1][ncars] = kx[1][0];

    for(n=0; n<ncars; n++){
      kx[2][n] = rk_x(x[n+1]+DT/2*kx[1][n+1], x[n]+DT/2*kx[1][n],
                      v[n]+DT/2*kv[1][n]);
      kv[2][n] = rk_v(x[n+1]+DT/2*kx[1][n+1], x[n]+DT/2*kx[1][n],
                      v[n]+DT/2*kv[1][n]);
    }
    kx[2][ncars] = kx[2][0];

    for(n=0; n<ncars; n++){
      kx[3][n] = rk_x(x[n+1]+DT*kx[2][n+1], x[n]+DT*kx[2][n],
                      v[n]+DT*kv[2][n]);
      kv[3][n] = rk_v(x[n+1]+DT*kx[2][n+1], x[n]+DT*kx[2][n],
                      v[n]+DT*kv[2][n]);
    }

    for(n=0; n<ncars; n++){
      x[n] = x[n] + DT/6*(kx[0][n] + 2*kx[1][n] + 2*kx[2][n] + kx[3][n]);
      v[n] = v[n] + DT/6*(kv[0][n] + 2*kv[1][n] + 2*kv[2][n] + kv[3][n]);
    }
    x[ncars] = x[0] + length;

    if(t%RECORD_POINTS_INTERVAL == 0){
      for(i=RECORD_POINTS-1; i>0; i--){
        phase_record[i][0] = phase_record[i-1][0];
        phase_record[i][1] = phase_record[i-1][1];
      }
      phase_record[0][0] = x[1]-x[0];
      phase_record[0][1] = v[0];
    }

    if(t%draw_interval == 0){
      g_sleep(wait_sec);
      g_new_frame();
      g_sel_scale(0);

      m_info = g_mouse();
      if(m_info.lbutton == G_YES){
        if(m_info.y > 0.0){
          draw_interval++;
        }
        else if(draw_interval>1){
          draw_interval--;
        }
      }

      g_line_color(G_BLACK);
      g_circle(0.0, 0.0, 1.0, G_YES, G_NO);
    
      g_line_color(G_RED);
      g_area_color(G_RED);
      for(n=0; n<ncars; n++){
        theta =(x[n]/length-(int)(x[n]/length))*2*M_PI;
        g_circle(cos(theta), sin(theta), 0.03, G_YES, G_YES);
//        g_marker(cos(theta), sin(theta));
      }
      sprintf(str, "OV model  cars=%d, average distance=%.1f", ncars, b);
      g_text(COORD_LEFT_MM1, COORD_TOP_MM1-20.0, str);
      sprintf(str, "A=%.2f, a=%.2f, b=%.2f, c=%.2f", A, ova, ovb, ovc);
      g_text(COORD_LEFT_MM1, COORD_TOP_MM1-10.0, str);

      g_sel_scale(1);
      g_line_color(G_BLACK);
      g_move(x_min_phase, y_max_phase);
      g_plot(x_min_phase, y_min_phase);
      g_plot(x_max_phase, y_min_phase);
      g_text_s(x_max_phase, 0.0, "x");
      g_text_s(0.0, y_max_phase, "v");


      g_move(0.0, ov(0.0));
      for(i=1; i<=NGRAPH; i++){
        g_plot(i*del_x_graph, ov(i*del_x_graph));
      }

      g_line_color(G_GREEN);
      g_move(phase_record[0][0], phase_record[0][1]);

      for(i=1; i<RECORD_POINTS; i++){
        g_plot(phase_record[i][0], phase_record[i][1]);
      }

      g_line_color(G_BLUE);
      g_area_color(G_BLUE);
      for(n=0; n<ncars; n++){
        g_circle(x[n+1]-x[n], v[n], 0.03, G_YES, G_YES);
      }
      g_show_frame();
//koma++;
//sprintf(str, "%05d.emf", koma);
//printf("%s\n", str);
//g_snapshot(str);
    }
  }

  g_sleep(-1.0);
  g_term();
}

double ov(double delx)
{
  return ova*(1/(1+exp(-ovb*(delx-ovc)))-1/(1+exp(ovb*ovc)));
//  return 0.4*(log(1+exp(8*delx-12))-log(1+exp(8*delx-15)));
}

double rk_x(double xn, double x, double v){
  return v;
}

double rk_v(double xn, double x, double v){
  return A*(ov(xn-x) - v);
}
